module GuessCL where

import Data.Char(isAlpha)
import Control.Exception(catch, IOException)
import System.Environment(getArgs)

import GuessGame

-- Render state as a string
render :: State-> String
render st =
   map (\c-> if elem c (hits st) then c else '_') (word st) ++ "\n"++
   show (length (misses st)) ++ " misses of "++ show numTries++ " tries."
  
-- Get the guess from the user (a single character)
getGuess :: State-> IO Char
getGuess st = do
  putStr $ "Your guess? "
  i<- getLine
  case i of
    "?" ->  do 
      putStrLn $ "Your misses so far: "++ misses st
      getGuess st
    [c] -> return c
    "" -> do
      putStrLn "No guess?"
      getGuess st
    _ -> do
      putStrLn "Please only guess at a time."
      getGuess st

-- Game main loop
play :: State-> IO ()
play st = do
  putStrLn (render st)
  c <- getGuess st
  case (processGuess c st) of
    (Hit, st)  -> play st
    (Miss, st) -> do putStrLn "Sorry, no."; play st
    (Repetition, st)-> do putStrLn "You already tried that."; play st    
    (GuessedIt, st)-> putStrLn "Congratulations, you guessed it."
    (TooManyTries, st) ->
      putStrLn $ "The word was "++ word st ++ " -- you lose."

-- Entry point: read dictionary, start game
main :: IO ()
main = 
  catch (do 
    as <- getArgs
    let df= case as of 
                ["de"] -> "dict-de_DE"
                ["en"] -> "dict-en_EN" 
                _ -> "dict-de_DE"
    dict <- readFile df 
    putStrLn $ "Selected dictionary "++ df ++ " ("++ show (length (lines dict))++ " entries)"
    init <- initialState (lines dict)
    putStrLn "Welcome to the Word Guessing Game!"
    play init)
    (\e-> putStrLn $ "Error: "++ show (e:: IOException))

