module GuessGUI where

import Data.IORef
import Data.Char(isAlpha)
import Control.Monad(when)
import Control.Exception(catch, IOException)
import System.Environment(getArgs)

import Graphics.UI.Gtk
import Graphics.UI.Gtk.Gdk.Events

import GuessGame


render :: State-> Label-> Label-> IO ()
render st l1 l2 = do
  labelSetText l1 $ map (\c-> if elem c (hits st) then c else '_') (word st)
  labelSetText l2 $ "Misses: "++ show (length (misses st)) ++ " out of "++ show numTries

message :: Window-> String-> Bool-> IO ()
message w msg quit = do 
  md <- messageDialogNew (Just w) [DialogModal, DialogDestroyWithParent] MessageInfo ButtonsClose msg
  dialogRun md
  widgetDestroy md -- Why is this needed?!
  when quit mainQuit

handleKeyPress :: Window-> Label-> Label-> IORef State-> Char-> IO ()
handleKeyPress w l1 l2 gs c = do
  st<- readIORef gs
  let (r, st')= processGuess c st
  writeIORef gs st'
  render st' l1 l2
  case r of
    Miss -> return () -- Do some blinking
    TooManyTries ->  message w ("Too many tries.\nThe word was "++ show (word st)++ "!\n\nYou lost.") True
    GuessedIt -> message w "Congratulations, you cracked it.\n\nYou won!"  True
    Repetition -> message w ("You already guessed `"++ [c]++ "`") False
    Hit -> return ()

run :: State-> IO ()
run st =  do
  initGUI
  window <- windowNew 
  hbox   <- hBoxNew True 10
  vbox   <- vBoxNew True 10
  l1     <- labelNew (Just (replicate 20 '*'))
  l2     <- labelNew (Nothing :: Maybe String)
  hb     <- buttonNewWithLabel "Hint"
  qb     <- buttonNewFromStock stockQuit -- WithLabel "Quit"
  set window [windowTitle := "Guess The Word!",
              windowDefaultWidth := 250, windowDefaultHeight := 200,
              containerBorderWidth := 10, containerChild := vbox]
  boxPackStart vbox l1 PackNatural 0
  boxPackStart vbox l2 PackNatural 0
  boxPackStart vbox hbox PackNatural 0
  boxPackStart hbox hb PackGrow 10
  boxPackStart hbox qb PackGrow 10

  -- Reference to hold game state
  gs <- newIORef st

  -- Process key presses
  onKeyPress window $ \e-> case eventKeyChar e of
    Just c-> do handleKeyPress window l1 l2 gs c; return True
    Nothing -> return True
  -- Process hint button
  onClicked hb $ do
    st<- readIORef gs
    message window ("\nYour guesses so far: "++ misses st) False

  -- Process quit button
  onClicked qb mainQuit

  -- Run it!
  onDestroy window mainQuit
  widgetShowAll window
  render st l1 l2
  mainGUI

main :: IO ()
main = catch (do
  as <- getArgs
  let df= case as of ["de"] -> "dict-de_DE"
                     ["en"] -> "dict-en_EN" 
                     _ -> "dict-de_DE"
  dict <- readFile df
  putStrLn $ "Selected dictionary "++ df ++ " ("++ show (length (lines dict))++ " entries)"
  st <- initialState (lines dict)
  run st)
  (\e-> putStrLn $ "Error: "++ show (e:: IOException))
